@@ -1,3 +1,5 @@
+## Ckmeans.1d.dp.R
+##
 ##   Ckmeans.1d.dp()-----Interface funcition to call C++ version kmeans.1d.dp()
 ##
 ##   Haizhou Wang
@@ -11,46 +13,11 @@
 #    May 17, 2016. MS
 #    September 25, 2016. MS. Introduced function ahist()
 
-## print method for Ckmeans.1d.dp object
-print.Ckmeans.1d.dp <- function(x, ...)
-{
-  with(x, {
-    cat("\nCluster means:\n")
-    print(centers, ...)
-    cat("\nCluster index associated with each element:\n")
-    print(cluster, ...)
-    cat("\nWithin-cluster sum of squares:\n")
-    print(withinss, ...)
-    if(length(size) > 1) {
-      cat("Ckmeans.1d.dp returns", length(size), "optimal clusters of sizes",
-          paste(size, collapse=", "), "\n")
-      cat("  minimum total within-cluster sum of squares:", tot.withinss, "\n")
-      cat("  maximum between-cluster sum of squares:", betweenss, "\n")
-      cat("  total sum of squares of input vector:", totss, "\n")
-      if(totss > 0) {
-        cat("  maximum (between-SS / total-SS):",
-            round(1000 * betweenss / totss)/10, "%\n")
-      }
-    } else {
-      cat("Ckmeans.1d.dp returns one cluster containing all elements.\n")
-    }
-  })
-
-  cat("\nAvailable components:\n")
-  print(names(x))
-  invisible(x)
-}
-
 ##Ckmeans.1d.dp : function which implement optimal one-dimensional clustering
 ## x is one-dimensional input vector
 ## k indicates cluster level
-Ckmeans.1d.dp <- function( x, k=c(1,9), y=1,
-                           method=c("linear", "loglinear", "quadratic"),
-                           estimate.k=c("BIC", "BIC 3.4.12") )
+cluster.1d.dp <- function( x, k, y, method, estimate.k, criterion, xname, yname )
 {
-  method <- match.arg(method)
-  estimate.k <- match.arg(estimate.k)
-
   if(is.null(k)) {
     k <- 1: min( 9, length(x) )
   } else {
@@ -97,19 +64,44 @@ Ckmeans.1d.dp <- function( x, k=c(1,9), y=1,
   clusters <- vector("integer", length(x))
   center <- vector("double", k.max)
   withinss <- vector("double", k.max)
-  size <- vector("integer", k.max)
+  size <- vector("double", k.max) # vector("integer", k.max)
   BIC <- vector("double", k.max-k.min+1)
 
   #Call external C++ function
-  result <- .C("Ckmeans_1d_dp", PACKAGE="Ckmeans.1d.dp",
-               data=as.double(x), length=as.integer(length(x)),
-               weight=as.double(y), weight_length=as.integer(length(y)),
-               Kmin=as.integer(k.min), Kmax=as.integer(k.max),
-               cluster=as.integer(clusters), centers=as.double(center),
-               withinss=as.double(withinss), size=as.integer(size),
-               BIC=as.double(BIC),
-               estimate.k=as.character(estimate.k),
-               method=as.character(method))
+  if(criterion == "L2") {
+    result <- .C("Ckmeans_1d_dp", PACKAGE="Ckmeans.1d.dp",
+                 data=as.double(x), length=as.integer(length(x)),
+                 weight=as.double(y), weight_length=as.integer(length(y)),
+                 Kmin=as.integer(k.min), Kmax=as.integer(k.max),
+                 cluster=as.integer(clusters), centers=as.double(center),
+                 withinss=as.double(withinss), size=as.double(size), # size=as.integer(size),
+                 BIC=as.double(BIC),
+                 estimate.k=as.character(estimate.k),
+                 method=as.character(method))
+    class <- "Ckmeans.1d.dp"
+  } else if (criterion == "L1") {
+    result <- .C("Ckmedian_1d_dp", PACKAGE="Ckmeans.1d.dp",
+                 data=as.double(x), length=as.integer(length(x)),
+                 weight=as.double(y), weight_length=as.integer(length(y)),
+                 Kmin=as.integer(k.min), Kmax=as.integer(k.max),
+                 cluster=as.integer(clusters), centers=as.double(center),
+                 withinss=as.double(withinss), size=as.double(size), # size=as.integer(size),
+                 BIC=as.double(BIC),
+                 estimate.k=as.character(estimate.k),
+                 method=as.character(method))
+    class <- "Ckmedian.1d.dp"
+  } else if(criterion == "L2Y") {
+    result <- .C("Cksegs_1d_dp", PACKAGE="Ckmeans.1d.dp",
+                 data=as.double(x), length=as.integer(length(x)),
+                 weight=as.double(y), weight_length=as.integer(length(y)),
+                 Kmin=as.integer(k.min), Kmax=as.integer(k.max),
+                 cluster=as.integer(clusters), centers=as.double(center),
+                 withinss=as.double(withinss), size=as.double(size), # size=as.integer(size),
+                 BIC=as.double(BIC),
+                 estimate.k=as.character(estimate.k),
+                 method=as.character(method))
+    class <- "Cksegs.1d.dp"
+  }
 
   if(length(result$cluster) > 0) {
     k.opt <- max(result$cluster) # length(unique(result$cluster))
@@ -127,8 +119,10 @@ Ckmeans.1d.dp <- function( x, k=c(1,9), y=1,
     }
   }
 
-  if(length(y) == length(x) && sum(y) != 0) {
+  if(length(y) == length(x) && sum(y) != 0 && criterion != "L2Y") {
     totss <- sum(y * (x - sum(as.numeric(x * y)) / sum(y))^2)
+  } else if(criterion == "L2Y") {
+    totss <- sum((y - sum(as.numeric(y)) / length(y))^2)
   } else {
     # totss <- sum(scale(x, scale=FALSE)^2) scale function is VERY SLOW!
     totss <- sum((x - sum(as.numeric(x)) / length(x))^2)
@@ -144,10 +138,45 @@ Ckmeans.1d.dp <- function( x, k=c(1,9), y=1,
       cluster = result$cluster, centers = result$centers[1:k.opt],
       withinss = result$withinss[1:k.opt], size = result$size[1:k.opt],
       totss = totss, tot.withinss = tot.withinss, betweenss = betweenss,
-      BIC = BIC, xname=deparse(substitute(x)),
-      yname=deparse(substitute(y))
+      BIC = BIC, xname=xname, yname=yname
     ),
-    class = "Ckmeans.1d.dp")
+    class = class)
 
   return( r )
-} ##end of Ckmeans.1d.dp()
+} ##end of cluster.1d.dp()
+
+
+Ckmeans.1d.dp <- function( x, k=c(1,9), y=1,
+                           method=c("linear", "loglinear", "quadratic"),
+                           estimate.k=c("BIC", "BIC 3.4.12") )
+{
+  method <- match.arg(method)
+  estimate.k <- match.arg(estimate.k)
+  cluster.1d.dp(x, k, y, method, estimate.k, "L2",
+                deparse(substitute(x)), deparse(substitute(y)))
+}
+
+Ckmedian.1d.dp <- function( x, k=c(1,9), y=1,
+                            method=c("linear", "loglinear", "quadratic"),
+                            estimate.k=c("BIC", "BIC 3.4.12") )
+{
+  method <- match.arg(method)
+  estimate.k <- match.arg(estimate.k)
+
+  if(length(y) > 1) {
+    warning("Ckmedian.1d.dp() ignores weight vector y and uses equal weight instead!")
+  }
+
+  cluster.1d.dp(x, k, y=1, method, estimate.k, "L1",
+                deparse(substitute(x)), deparse(substitute(y)))
+}
+
+Cksegs.1d.dp <- function(y, k=c(1,9), x=seq_along(y),
+                         method=c("quadratic", "linear", "loglinear"),
+                         estimate.k=c("BIC", "BIC 3.4.12") )
+{
+  method <- match.arg(method)
+  estimate.k <- match.arg(estimate.k)
+  cluster.1d.dp(x, k, y, method, estimate.k, "L2Y",
+                deparse(substitute(x)), deparse(substitute(y)))
+}
