@@ -30,6 +30,7 @@
  to search for. Made other changes.
  March 29, 2014. Haizhou Wang. Replaced "int *Ks" and "int *nK" by
  "int *minK" and "int *maxK".
+ May 29, 2017. Joe Song. Change size from integer to double.
  */
 
 #include <string>
@@ -48,12 +49,12 @@ extern "C" {
    cluster:  An array of cluster IDs for each point in x.
    centers:  An array of centers for each cluster.
    withinss: An array of within-cluster sum of squares for each cluster.
-   size:     An array of sizes of each cluster.
+   size:     An array of (weighted) sizes of each cluster.
    */
 
   void Ckmeans_1d_dp(double *x, int* length, double *y, int * ylength,
                      int* minK, int *maxK, int* cluster,
-                     double* centers, double* withinss, int* size,
+                     double* centers, double* withinss, double * size, // int* size,
                      double* BICs,
                      char ** estimate_k, char ** method)
   {
@@ -64,33 +65,84 @@ extern "C" {
 
     kmeans_1d_dp(x, (size_t)*length, y, (size_t)(*minK), (size_t)(*maxK),
                  cluster, centers, withinss, size, BICs,
-                 std::string(estimate_k[0]), std::string(method[0]));
+                 std::string(estimate_k[0]), std::string(method[0]),
+                 L2);
 
     // Change the cluster numbering from 0-based to 1-based
     for(size_t i=0; i< *length; ++i) {
       cluster[i] ++;
     }
   }
+
+  void Ckmedian_1d_dp(double *x, int* length, double *y, int * ylength,
+                      int* minK, int *maxK, int* cluster,
+                      double* centers, double* withinss, double * size, // int* size,
+                      double* BICs,
+                      char ** estimate_k, char ** method)
+  {
+    // std::cout << method[0] << std::endl;
+
+    // Call C++ version one-dimensional clustering algorithm*/
+    if(*ylength != *length) { y = 0; }
+
+    kmeans_1d_dp(x, (size_t)*length, y,
+                 (size_t)(*minK), (size_t)(*maxK),
+                 cluster, centers, withinss, size, BICs,
+                 std::string(estimate_k[0]), std::string(method[0]),
+                 L1);
+
+    // Change the cluster numbering from 0-based to 1-based
+    for(size_t i=0; i< *length; ++i) {
+      cluster[i] ++;
+    }
+  }
+
+  void Cksegs_1d_dp(double *x, int* length, double *y, int * ylength,
+                    int* minK, int *maxK, int* cluster,
+                    double* centers, double* withinss, double * size, // int* size,
+                    double* BICs,
+                    char ** estimate_k, char ** method)
+  {
+    // std::cout << method[0] << std::endl;
+
+    // Call C++ version one-dimensional clustering algorithm*/
+    if(*ylength != *length) { y = 0; }
+
+    kmeans_1d_dp(x, (size_t)*length, y, (size_t)(*minK), (size_t)(*maxK),
+                 cluster, centers, withinss, size, BICs,
+                 std::string(estimate_k[0]), std::string(method[0]),
+                 L2Y);
+
+    // Change the cluster numbering from 0-based to 1-based
+    for(size_t i=0; i< *length; ++i) {
+      cluster[i] ++;
+    }
+  }
+
 } // End of extern "C"
 
 
 static const
   R_CMethodDef cMethods[] = {
     {"Ckmeans_1d_dp",  (DL_FUNC) & Ckmeans_1d_dp, 13},
+    {"Ckmeans_1d_dp",  (DL_FUNC) & Ckmedian_1d_dp, 13},
+    {"Ckmeans_1d_dp",  (DL_FUNC) & Cksegs_1d_dp, 13},
     {NULL}
   };
 
 static const
   R_CallMethodDef callMethods[] = {
     {"Ckmeans_1d_dp", (DL_FUNC) & Ckmeans_1d_dp, 13},
+    {"Ckmeans_1d_dp", (DL_FUNC) & Ckmedian_1d_dp, 13},
+    {"Ckmeans_1d_dp", (DL_FUNC) & Cksegs_1d_dp, 13},
     {NULL} };
 
 void R_init_Ckmeans_1d_dp(DllInfo *info)
 {
   /* Register the .C and .Call routines.
-  No .Fortran() or .External() routines,
-  so pass those arrays as NULL.
-  */
+   No .Fortran() or .External() routines,
+   so pass those arrays as NULL.
+   */
   R_registerRoutines(info,
                      cMethods, callMethods,
                      NULL, NULL);
